@@ -12,6 +12,7 @@ import copy
 import logging
 import os
 import re
+from hashlib import sha256
 from pathlib import Path
 
 from .defaults import DEFAULT_CHECKOUT_METHOD
@@ -179,7 +180,8 @@ class Project:
         """
         Return the dependencies of a checked out project using the information
         retrieved from the configuration files.
-        @return: list of used projects (all converted to lowercase)
+        @return: list of used projects (all converted to lowercase, unless we are
+        in a slot, in which case the actual casing is used)
         """
         proj_root = self.baseDir
 
@@ -305,7 +307,7 @@ class Project:
             except:
                 pass
         else:
-            __log__.warning("cannot discover dependencies for %s", self)
+            __log__.debug("cannot discover dependencies for %s", self)
 
         deps = sorted(set(deps + self._deps))
         if self.slot:
@@ -589,6 +591,44 @@ class Project:
         self._fixCMT(patchfile, dryrun=dryrun)
         self._fixProjectConfigJSON(patchfile, dryrun=dryrun)
 
+    def hash(self, platform: str = "") -> str:
+        """
+        Compute a unique id for the specific project version of the code.
+
+        The hash is a `sha256` sum of the commit ids used for the checkout
+        of the project, unless the project is disabled, in which case we use
+        only name and version.
+
+        If `platform` is specified, it is included in the hash and if the
+        project is attached to a slot and the dependencies are taken into
+        account too.
+        """
+        # hash is computed as sha256 sum of
+        # - commit or id
+        # - 'mr_commit's
+        hash = sha256()
+
+        if self.disabled:
+            hash.update(f"{self.name}/{self.version}".encode())
+        else:
+            hash.update((self.checkout_opts.get("commit") or self.id()).encode())
+            for _, mr_commit in self.checkout_opts.get("merges", []):
+                hash.update(mr_commit.encode())
+
+        if platform:
+            # include the platform in the hash
+            hash.update(platform.encode())
+            if self.slot:
+                # dependencies are considered if we are in a slot
+                for dep in [
+                    getattr(self.slot, dep)
+                    for dep in self.dependencies()
+                    if hasattr(self.slot, dep)
+                ]:
+                    hash.update(dep.hash(platform).encode())
+
+        return hash.hexdigest()
+
     def artifacts(self, stage, platform=""):
         """
         Returns the name of the artifacts zip file for a project.
@@ -612,23 +652,9 @@ class Project:
             raise ValueError("Platform must be set for build and test")
 
         if stage == "checkout":
-            from hashlib import sha256
-            from pathlib import Path
-
             path = Path(stage)
-
-            # hash is computed as sha256 sum of
-            # - commit or id
-            # - 'mr_commit's
-            hash = sha256()
-
-            hash.update((self.checkout_opts.get("commit") or self.id()).encode())
-            for _, mr_commit in self.checkout_opts.get("merges", []):
-                hash.update(mr_commit.encode())
-
-            digest = hash.hexdigest()
-
-            return str(path / self.name / digest[:2] / (digest + ".zip"))
+            hash = self.hash()
+            return str(path / self.name / hash[:2] / (hash + ".zip"))
 
         pid = self.id()
         if self.slot:
@@ -794,6 +820,28 @@ class Package:
             else str(self)
         )
 
+    def hash(self) -> str:
+        """
+        Compute a unique id for the specific project version of the code.
+
+        The hash is a `sha256` sum of the commit ids used for the checkout
+        of the data package.
+        """
+        # hash is computed as sha256 sum of
+        # - version name (as it affects the content of the checkout) + commit
+        #   - or id
+        # - 'mr_commit's
+        hash = sha256()
+        if "commit" in self.checkout_opts:
+            hash.update(self.version.encode())
+            hash.update(self.checkout_opts["commit"].encode())
+        else:
+            hash.update(self.id().encode())
+        for _, mr_commit in self.checkout_opts.get("merges", []):
+            hash.update(mr_commit.encode())
+
+        return hash.hexdigest()
+
     def artifacts(self, stage="checkout", platform=""):
         """
         Returns the name of the artifacts zip file for a package.
@@ -809,24 +857,14 @@ class Package:
         if stage != "checkout" or platform:
             raise ValueError("Only 'checkout' stage allowed for DataProject.artifacts")
 
-        from hashlib import sha256
-        from pathlib import Path
-
         path = Path(stage)
+        hash = self.hash()
+        return str(path / self.name / hash[:2] / (hash + ".zip"))
 
-        # hash is computed as sha256 sum of
-        # - for each package
-        #   - commit or id
-        #   - 'mr_commit's
-        hash = sha256()
+    def get_deployment_directory(self) -> Path:
+        from . import lbnightly_settings
 
-        hash.update((self.checkout_opts.get("commit") or self.id()).encode())
-        for _, mr_commit in self.checkout_opts.get("merges", []):
-            hash.update(mr_commit.encode())
-
-        digest = hash.hexdigest()
-
-        return str(path / self.name / digest[:2] / (digest + ".zip"))
+        return Path(lbnightly_settings().installations.path) / self.id()
 
 
 class PackagesList(_ContainedList):
@@ -917,6 +955,25 @@ class DataProject(Project):
                 "%r object has no attribute %r" % (self.__class__.__name__, name)
             )
 
+    def hash(self, platform: str = "") -> str:
+        """
+        Compute a unique id for the specific project version of the code.
+
+        The hash is a `sha256` sum of the hashes of the contained packages.
+
+        The `platform` argument is ignored but kept for compatibility with
+        he base class.
+        """
+        # hash is computed as sha256 sum of
+        # - for each package
+        #   - package hash
+        hash = sha256()
+
+        for package in sorted(self.packages, key=lambda p: p.name):
+            hash.update(package.hash().encode())
+
+        return hash.hexdigest()
+
     def artifacts(self, stage="checkout", platform=""):
         """
         Returns the name of the artifacts zip file for a data project.
@@ -932,25 +989,10 @@ class DataProject(Project):
         if stage != "checkout" or platform:
             raise ValueError("Only 'checkout' stage allowed for DataProject.artifacts")
 
-        from hashlib import sha256
-        from pathlib import Path
-
         path = Path(stage)
+        hash = self.hash()
 
-        # hash is computed as sha256 sum of
-        # - for each package
-        #   - commit or id
-        #   - 'mr_commit's
-        hash = sha256()
-
-        for package in self.packages:
-            hash.update((package.checkout_opts.get("commit") or package.id()).encode())
-            for _, mr_commit in package.checkout_opts.get("merges", []):
-                hash.update(mr_commit.encode())
-
-        digest = hash.hexdigest()
-
-        return str(path / self.name / digest[:2] / (digest + ".zip"))
+        return str(path / self.name / hash[:2] / (hash + ".zip"))
 
 
 class DBASE(DataProject):
