@@ -199,9 +199,17 @@ class Slot:
             )
             container.packages.append(pkg)
 
+        projects = [
+            Project.fromDict(prj)
+            if prj["name"] not in containers
+            else containers[prj["name"]]
+            for prj in data.get("projects", [])
+        ]
+        if containers and not projects:
+            projects = list(containers.values())
         slot = cls(
             name=data.get("slot", None),
-            projects=list(containers.values()),
+            projects=projects,
             env=data.get("env", []),
             desc=data.get("description"),
             disabled=data.get("disabled", False),
@@ -215,14 +223,6 @@ class Slot:
         if "build_tool" in data:
             slot.build_tool = data["build_tool"]
 
-        slot.projects.extend(
-            [
-                Project.fromDict(prj)
-                for prj in data.get("projects", [])
-                if prj["name"] not in containers
-            ]
-        )
-
         slot.error_exceptions = data.get("error_exceptions", [])
         slot.warning_exceptions = data.get("warning_exceptions", [])
         slot.preconditions = data.get("preconditions", [])
@@ -276,6 +276,8 @@ class Slot:
             else containers[prj["name"]]
             for prj in data.get("projects", [])
         ]
+        if containers and not projects:
+            projects = list(containers.values())
         self.projects = ProjectsList(self, projects)
         self.env = data.get("env", [])
         self.desc = data.get("description")
