@@ -188,6 +188,16 @@ class _ContainedList(object):
             raise ValueError(msg)
         return element
 
+    def _assertUnique(self, element, iterable=None):
+        """
+        Ensure that the there is only one instance of the project.
+        """
+        for item in iterable or self:
+            if element.toDict() == item.toDict():
+                raise ValueError(
+                    f"There is already project {element} added to the slot."
+                )
+
     def __init__(self, container, iterable=None):
         """
         Initialize the list from an optional iterable, which must contain
@@ -198,7 +208,12 @@ class _ContainedList(object):
             self._elements = []
         else:
             self._elements = list(map(self._assertType, iterable))
-            for element in self._elements:
+            for index, element in enumerate(self._elements):
+                remainder = [
+                    item for idx, item in enumerate(self._elements) if idx != index
+                ]
+                if remainder:
+                    self._assertUnique(element, remainder)
                 setattr(element, self.__container_member__, self.container)
 
     def __eq__(self, other):
@@ -268,6 +283,7 @@ class _ContainedList(object):
         Item insertion that binds the added object to the container.
         """
         self._assertType(element)
+        self._assertUnique(element)
         setattr(element, self.__container_member__, self.container)
         return self._elements.insert(idx, element)
 
@@ -276,6 +292,7 @@ class _ContainedList(object):
         Item insertion that binds the added object to the container.
         """
         self._assertType(element)
+        self._assertUnique(element)
         setattr(element, self.__container_member__, self.container)
         return self._elements.append(element)
 
